import argparse
import os
from .utils import get_home
import pathlib
import logging
import subprocess

logging.basicConfig(level=logging.INFO)

from .up import main as up
from .status import main as status
from .down import main as down
from .shell import main as shell

# os.chdir("/Users/yy/work/huiyin/xud-docker-dev/xud-testnet")
# os.system("docker-compose up -d")
#
# time.sleep(3)
#
# print(w3.eth.blockNumber)
# print(w3.eth.syncing)
# os.system("docker-compose down")
# os.system("docker-compose up -d geth")
# os.system("docker-compose exec -T geth geth --testnet --exec 'admin.nodeInfo' attach")
# os.system("docker-compose exec -T geth geth --testnet --exec 'eth.accounts' attach")
# os.system("docker-compose exec -T geth geth --testnet --exec 'eth.syncing' attach")
# os.system("docker-compose exec -T geth geth --testnet --exec 'eth.blockNumber' attach")
# os.system("docker-compose exec -T geth geth --testnet account list")
# os.system("docker-compose logs --tail=200 geth")
# output = subprocess.check_output("pwd")
# print(output)
# output = subprocess.check_output("docker-compose exec geth geth --testnet --help", shell=True)
# print(output)
# docker-compose exec geth geth --testnet attach
# personal.createAccount()


def check_docker():
    p = subprocess.Popen('which docker', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    r = p.returncode
    if r != 0:
        logging.error("docker is missing")
        exit(1)
    # os.system('docker --version')


def check_docker_compose():
    p = subprocess.Popen('which docker', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    r = p.returncode
    if r != 0:
        logging.error("docker-compose is missing")
        exit(1)
    # os.system('docker-compose --version')


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-n", "--network", type=str, default='testnet', choices=['regtest', 'simnet', 'testnet', 'mainnet'])

    sub = parser.add_subparsers(dest="subparser")
    cmd_up = sub.add_parser("up")
    cmd_down = sub.add_parser("down")
    cmd_status = sub.add_parser("status")
    cmd_shell = sub.add_parser("shell")

    args = parser.parse_args()
    kwargs = vars(args)

    # Make sure docker is installed
    check_docker()

    # Make sure docker-compose is installed
    check_docker_compose()

    home = get_home() + "/" + args.network

    if not os.path.exists(home):
        pathlib.Path(home).mkdir(parents=True, exist_ok=True)

    if not os.path.isdir(home):
        logging.error("{} is not a directory", home)
        exit(1)

    # Go to xud-docker home directory
    os.chdir(home)

    globals()[kwargs.pop('subparser')](**kwargs)


if __name__ == '__main__':
    main()



import grpc
import codecs
from . import lnrpc_pb2 as ln
from .lnrpc_pb2_grpc import LightningStub
import logging


class Lnd:
    def __init__(self, connstr, cert, macaroon):
        macaroon_bytes = macaroon
        macaroon = codecs.encode(macaroon_bytes, 'hex')

        def metadata_callback(context, callback):
            callback([('macaroon', macaroon)], None)

        cert_creds = grpc.ssl_channel_credentials(cert)
        auth_creds = grpc.metadata_call_credentials(metadata_callback)
        combined_creds = grpc.composite_channel_credentials(cert_creds, auth_creds)
        channel = grpc.secure_channel(connstr, combined_creds)
        self.client = LightningStub(channel)

    @property
    def status(self):
        try:
            info = self.client.GetInfo(ln.GetInfoRequest())
            if info.synced_to_chain:
                return "OK"
            return "Syncing"
        except:
            logging.exception('Lnd failed to get status')
            return "Error"



import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(
    name="xud-wizard",
    version="0.1.3",
    scripts=[],
    author="Yang Yang",
    author_email="reliveyy@gmail.com",
    description="A wrapper program for xud-docker",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    entry_points={
        "console_scripts": [
            # "xud-wizard = xud-wizard.__main__:main",
            # This will not work because of the dash(-) between the package name
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "pexpect==4.7.0",
        "web3==4.9.2",
        "python-bitcoinrpc==1.0",
        "grpcio==1.20.1",
        "googleapis-common-protos==1.6.0",
        "fabric==2.4.0",
    ],

)



import os
import logging
import subprocess
import re


def remote_network_by_id(network_id):
    p = subprocess.Popen('docker network rm ' + network_id, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    r = p.returncode
    if r != 0:
        logging.error("Failed to remove docker network: %s", network_id)
        exit(1)


def remove_ambiguous_network(network_name):
    p = subprocess.Popen('docker network ls | grep ' + network_name, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    r = p.returncode
    if r != 0:
        logging.error("Failed to get all docker network named: %s", network_name)
        exit(1)
    for line in stdout.decode().splitlines():
        network_id = line.split()[0]
        remote_network_by_id(network_id)


def docker_compose_down():
    p = subprocess.Popen('docker-compose down', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    r = p.returncode
    if r != 0:
        for line in stderr.decode().splitlines():
            m = re.search(r'(\d+) matches found based on name: network (.+) is ambiguous', line)
            if m is not None:
                network_name = m.group(2)
                remove_ambiguous_network(network_name)
                docker_compose_down()
            else:
                exit(1)


def main(network):
    if os.path.exists("docker-compose.yml"):
        logging.info("Clean up the {} environment".format(network))
        docker_compose_down()
